import { Http,Util } from '@/utils';
import EntityService from '../entity-service';



/**
 * 潜在顾客服务对象基类
 *
 * @export
 * @class LeadServiceBase
 * @extends {EntityServie}
 */
export default class LeadServiceBase extends EntityService {

    /**
     * Creates an instance of  LeadServiceBase.
     * 
     * @param {*} [opts={}]
     * @memberof  LeadServiceBase
     */
    constructor(opts: any = {}) {
        super(opts);
    }

    /**
     * 初始化基础数据
     *
     * @memberof LeadServiceBase
     */
    public initBasicData(){
        this.APPLYDEKEY ='lead';
        this.APPDEKEY = 'leadid';
        this.APPDENAME = 'leads';
        this.APPDETEXT = 'fullname';
        this.APPNAME = 'businesscentral';
        this.SYSTEMNAME = 'ibizbusinesscentral';
    }

// 实体接口

    /**
     * Select接口方法
     *
     * @param {*} [context={}]
     * @param {*} [data={}]
     * @param {boolean} [isloading]
     * @returns {Promise<any>}
     * @memberof LeadServiceBase
     */
    public async Select(context: any = {},data: any = {}, isloading?: boolean): Promise<any> {
        if(context.contact && context.lead){
            return Http.getInstance().get(`/contacts/${context.contact}/leads/${context.lead}/select`,isloading);
        }
        if(context.campaign && context.lead){
            return Http.getInstance().get(`/campaigns/${context.campaign}/leads/${context.lead}/select`,isloading);
        }
            return Http.getInstance().get(`/leads/${context.lead}/select`,isloading);
    }

    /**
     * Update接口方法
     *
     * @param {*} [context={}]
     * @param {*} [data={}]
     * @param {boolean} [isloading]
     * @returns {Promise<any>}
     * @memberof LeadServiceBase
     */
    public async Update(context: any = {},data: any = {}, isloading?: boolean): Promise<any> {
        if(context.contact && context.lead){
            return Http.getInstance().put(`/contacts/${context.contact}/leads/${context.lead}`,data,isloading);
        }
        if(context.campaign && context.lead){
            return Http.getInstance().put(`/campaigns/${context.campaign}/leads/${context.lead}`,data,isloading);
        }
        let masterData:any = {};
        let listleadsData:any = [];
        if(!Object.is(this.tempStorage.getItem(context.srfsessionkey+'_listleads'),'undefined')){
            listleadsData = JSON.parse(this.tempStorage.getItem(context.srfsessionkey+'_listleads') as any);
            if(listleadsData && listleadsData.length && listleadsData.length > 0){
                listleadsData.forEach((item:any) => {
                    if(item.srffrontuf){
                        if(Object.is(item.srffrontuf,"0")){
                            item.relationshipsid = null;
                        }
                        delete item.srffrontuf;
                    }
                });
            }
        }
        masterData.listleads = listleadsData;
        Object.assign(data,masterData);
            let res:any = await  Http.getInstance().put(`/leads/${context.lead}`,data,isloading);
            this.tempStorage.setItem(context.srfsessionkey+'_listleads',JSON.stringify(res.data.listleads));
            return res;
    }

    /**
     * GetDraft接口方法
     *
     * @param {*} [context={}]
     * @param {*} [data={}]
     * @param {boolean} [isloading]
     * @returns {Promise<any>}
     * @memberof LeadServiceBase
     */
    public async GetDraft(context: any = {},data: any = {}, isloading?: boolean): Promise<any> {
        if(context.contact && true){
            return Http.getInstance().get(`/contacts/${context.contact}/leads/getdraft`,isloading);
        }
        if(context.campaign && true){
            return Http.getInstance().get(`/campaigns/${context.campaign}/leads/getdraft`,isloading);
        }
        let res:any = await  Http.getInstance().get(`/leads/getdraft`,isloading);
        res.data.lead = data.lead;
            this.tempStorage.setItem(context.srfsessionkey+'_listleads',JSON.stringify(res.data.listleads));
        return res;
    }

    /**
     * CheckKey接口方法
     *
     * @param {*} [context={}]
     * @param {*} [data={}]
     * @param {boolean} [isloading]
     * @returns {Promise<any>}
     * @memberof LeadServiceBase
     */
    public async CheckKey(context: any = {},data: any = {}, isloading?: boolean): Promise<any> {
        if(context.contact && context.lead){
            return Http.getInstance().post(`/contacts/${context.contact}/leads/${context.lead}/checkkey`,data,isloading);
        }
        if(context.campaign && context.lead){
            return Http.getInstance().post(`/campaigns/${context.campaign}/leads/${context.lead}/checkkey`,data,isloading);
        }
            return Http.getInstance().post(`/leads/${context.lead}/checkkey`,data,isloading);
    }

    /**
     * Get接口方法
     *
     * @param {*} [context={}]
     * @param {*} [data={}]
     * @param {boolean} [isloading]
     * @returns {Promise<any>}
     * @memberof LeadServiceBase
     */
    public async Get(context: any = {},data: any = {}, isloading?: boolean): Promise<any> {
        if(context.contact && context.lead){
            return Http.getInstance().get(`/contacts/${context.contact}/leads/${context.lead}`,isloading);
        }
        if(context.campaign && context.lead){
            return Http.getInstance().get(`/campaigns/${context.campaign}/leads/${context.lead}`,isloading);
        }
            let res:any = await Http.getInstance().get(`/leads/${context.lead}`,isloading);
            this.tempStorage.setItem(context.srfsessionkey+'_listleads',JSON.stringify(res.data.listleads));
            return res;

    }

    /**
     * Save接口方法
     *
     * @param {*} [context={}]
     * @param {*} [data={}]
     * @param {boolean} [isloading]
     * @returns {Promise<any>}
     * @memberof LeadServiceBase
     */
    public async Save(context: any = {},data: any = {}, isloading?: boolean): Promise<any> {
        if(context.contact && context.lead){
            return Http.getInstance().post(`/contacts/${context.contact}/leads/${context.lead}/save`,data,isloading);
        }
        if(context.campaign && context.lead){
            return Http.getInstance().post(`/campaigns/${context.campaign}/leads/${context.lead}/save`,data,isloading);
        }
        let masterData:any = {};
        let listleadsData:any = [];
        if(!Object.is(this.tempStorage.getItem(context.srfsessionkey+'_listleads'),'undefined')){
            listleadsData = JSON.parse(this.tempStorage.getItem(context.srfsessionkey+'_listleads') as any);
            if(listleadsData && listleadsData.length && listleadsData.length > 0){
                listleadsData.forEach((item:any) => {
                    if(item.srffrontuf){
                        if(Object.is(item.srffrontuf,"0")){
                            item.relationshipsid = null;
                        }
                        delete item.srffrontuf;
                    }
                });
            }
        }
        masterData.listleads = listleadsData;
        Object.assign(data,masterData);
            let res:any = await  Http.getInstance().post(`/leads/${context.lead}/save`,data,isloading);
            this.tempStorage.setItem(context.srfsessionkey+'_listleads',JSON.stringify(res.data.listleads));
            return res;
    }

    /**
     * Create接口方法
     *
     * @param {*} [context={}]
     * @param {*} [data={}]
     * @param {boolean} [isloading]
     * @returns {Promise<any>}
     * @memberof LeadServiceBase
     */
    public async Create(context: any = {},data: any = {}, isloading?: boolean): Promise<any> {
        if(context.contact && true){
            if(!data.srffrontuf || data.srffrontuf !== "1"){
                data[this.APPDEKEY] = null;
            }
            if(data.srffrontuf){
                delete data.srffrontuf;
            }
            return Http.getInstance().post(`/contacts/${context.contact}/leads`,data,isloading);
        }
        if(context.campaign && true){
            if(!data.srffrontuf || data.srffrontuf !== "1"){
                data[this.APPDEKEY] = null;
            }
            if(data.srffrontuf){
                delete data.srffrontuf;
            }
            return Http.getInstance().post(`/campaigns/${context.campaign}/leads`,data,isloading);
        }
        let masterData:any = {};
        let listleadsData:any = [];
        if(!Object.is(this.tempStorage.getItem(context.srfsessionkey+'_listleads'),'undefined')){
            listleadsData = JSON.parse(this.tempStorage.getItem(context.srfsessionkey+'_listleads') as any);
            if(listleadsData && listleadsData.length && listleadsData.length > 0){
                listleadsData.forEach((item:any) => {
                    if(item.srffrontuf){
                        if(Object.is(item.srffrontuf,"0")){
                            item.relationshipsid = null;
                        }
                        delete item.srffrontuf;
                    }
                });
            }
        }
        masterData.listleads = listleadsData;
        Object.assign(data,masterData);
        if(!data.srffrontuf || data.srffrontuf !== "1"){
            data[this.APPDEKEY] = null;
        }
        if(data.srffrontuf){
            delete data.srffrontuf;
        }
        let tempContext:any = JSON.parse(JSON.stringify(context));
        let res:any = await Http.getInstance().post(`/leads`,data,isloading);
        this.tempStorage.setItem(tempContext.srfsessionkey+'_listleads',JSON.stringify(res.data.listleads));
        return res;
    }

    /**
     * Remove接口方法
     *
     * @param {*} [context={}]
     * @param {*} [data={}]
     * @param {boolean} [isloading]
     * @returns {Promise<any>}
     * @memberof LeadServiceBase
     */
    public async Remove(context: any = {},data: any = {}, isloading?: boolean): Promise<any> {
        if(context.contact && context.lead){
            return Http.getInstance().delete(`/contacts/${context.contact}/leads/${context.lead}`,isloading);
        }
        if(context.campaign && context.lead){
            return Http.getInstance().delete(`/campaigns/${context.campaign}/leads/${context.lead}`,isloading);
        }
            return Http.getInstance().delete(`/leads/${context.lead}`,isloading);

    }

    /**
     * FetchDefault接口方法
     *
     * @param {*} [context={}]
     * @param {*} [data={}]
     * @param {boolean} [isloading]
     * @returns {Promise<any>}
     * @memberof LeadServiceBase
     */
    public async FetchDefault(context: any = {},data: any = {}, isloading?: boolean): Promise<any> {
        if(context.contact && true){
            let tempData:any = JSON.parse(JSON.stringify(data));
            return Http.getInstance().get(`/contacts/${context.contact}/leads/fetchdefault`,tempData,isloading);
        }
        if(context.campaign && true){
            let tempData:any = JSON.parse(JSON.stringify(data));
            return Http.getInstance().get(`/campaigns/${context.campaign}/leads/fetchdefault`,tempData,isloading);
        }
        let tempData:any = JSON.parse(JSON.stringify(data));
        return Http.getInstance().get(`/leads/fetchdefault`,tempData,isloading);
    }
}