package cn.ibizlab.businesscentral.core.base.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.base.domain.Audit;
import cn.ibizlab.businesscentral.core.base.filter.AuditSearchContext;
import cn.ibizlab.businesscentral.core.base.service.IAuditService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.base.mapper.AuditMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[审核] 服务对象接口实现
 */
@Slf4j
@Service("AuditServiceImpl")
public class AuditServiceImpl extends ServiceImpl<AuditMapper, Audit> implements IAuditService {


    private int batchSize = 500;

    @Override
    @Transactional
    public boolean create(Audit et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getAuditid()),et);
        return true;
    }

    @Override
    public void createBatch(List<Audit> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    public Audit getDraft(Audit et) {
        return et;
    }

    @Override
    @Transactional
    public Audit get(String key) {
        Audit et = getById(key);
        if(et==null){
            et=new Audit();
            et.setAuditid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean update(Audit et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("auditid",et.getAuditid())))
            return false;
        CachedBeanCopier.copy(get(et.getAuditid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<Audit> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean save(Audit et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(Audit et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<Audit> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<Audit> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    public boolean checkKey(Audit et) {
        return (!ObjectUtils.isEmpty(et.getAuditid()))&&(!Objects.isNull(this.getById(et.getAuditid())));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<Audit> searchDefault(AuditSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<Audit> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<Audit>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }





    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<Audit> getAuditByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<Audit> getAuditByEntities(List<Audit> entities) {
        List ids =new ArrayList();
        for(Audit entity : entities){
            Serializable id=entity.getAuditid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



