package cn.ibizlab.businesscentral.core.base.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.base.domain.Feedback;
import cn.ibizlab.businesscentral.core.base.filter.FeedbackSearchContext;
import cn.ibizlab.businesscentral.core.base.service.IFeedbackService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.base.mapper.FeedbackMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[反馈] 服务对象接口实现
 */
@Slf4j
@Service("FeedbackServiceImpl")
public class FeedbackServiceImpl extends ServiceImpl<FeedbackMapper, Feedback> implements IFeedbackService {

    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.IContactService contactService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.ITransactionCurrencyService transactioncurrencyService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean update(Feedback et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("feedbackid",et.getFeedbackid())))
            return false;
        CachedBeanCopier.copy(get(et.getFeedbackid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<Feedback> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean create(Feedback et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getFeedbackid()),et);
        return true;
    }

    @Override
    public void createBatch(List<Feedback> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public Feedback get(String key) {
        Feedback et = getById(key);
        if(et==null){
            et=new Feedback();
            et.setFeedbackid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public Feedback getDraft(Feedback et) {
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    public boolean checkKey(Feedback et) {
        return (!ObjectUtils.isEmpty(et.getFeedbackid()))&&(!Objects.isNull(this.getById(et.getFeedbackid())));
    }
    @Override
    @Transactional
    public boolean save(Feedback et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(Feedback et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<Feedback> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<Feedback> list) {
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<Feedback> selectByCreatedbycontact(String contactid) {
        return baseMapper.selectByCreatedbycontact(contactid);
    }

    @Override
    public void removeByCreatedbycontact(String contactid) {
        this.remove(new QueryWrapper<Feedback>().eq("createdbycontact",contactid));
    }

	@Override
    public List<Feedback> selectByCreatedonbehalfbycontact(String contactid) {
        return baseMapper.selectByCreatedonbehalfbycontact(contactid);
    }

    @Override
    public void removeByCreatedonbehalfbycontact(String contactid) {
        this.remove(new QueryWrapper<Feedback>().eq("createdonbehalfbycontact",contactid));
    }

	@Override
    public List<Feedback> selectByTransactioncurrencyid(String transactioncurrencyid) {
        return baseMapper.selectByTransactioncurrencyid(transactioncurrencyid);
    }

    @Override
    public void removeByTransactioncurrencyid(String transactioncurrencyid) {
        this.remove(new QueryWrapper<Feedback>().eq("transactioncurrencyid",transactioncurrencyid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<Feedback> searchDefault(FeedbackSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<Feedback> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<Feedback>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }





    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<Feedback> getFeedbackByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<Feedback> getFeedbackByEntities(List<Feedback> entities) {
        List ids =new ArrayList();
        for(Feedback entity : entities){
            Serializable id=entity.getFeedbackid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



