package cn.ibizlab.businesscentral.core.base.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.base.domain.PersonalDocumentTemplate;
import cn.ibizlab.businesscentral.core.base.filter.PersonalDocumentTemplateSearchContext;
import cn.ibizlab.businesscentral.core.base.service.IPersonalDocumentTemplateService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.base.mapper.PersonalDocumentTemplateMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[个人文档模板] 服务对象接口实现
 */
@Slf4j
@Service("PersonalDocumentTemplateServiceImpl")
public class PersonalDocumentTemplateServiceImpl extends ServiceImpl<PersonalDocumentTemplateMapper, PersonalDocumentTemplate> implements IPersonalDocumentTemplateService {


    private int batchSize = 500;

    @Override
    @Transactional
    public PersonalDocumentTemplate get(String key) {
        PersonalDocumentTemplate et = getById(key);
        if(et==null){
            et=new PersonalDocumentTemplate();
            et.setPersonaldocumenttemplateid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean update(PersonalDocumentTemplate et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("personaldocumenttemplateid",et.getPersonaldocumenttemplateid())))
            return false;
        CachedBeanCopier.copy(get(et.getPersonaldocumenttemplateid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<PersonalDocumentTemplate> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    public boolean checkKey(PersonalDocumentTemplate et) {
        return (!ObjectUtils.isEmpty(et.getPersonaldocumenttemplateid()))&&(!Objects.isNull(this.getById(et.getPersonaldocumenttemplateid())));
    }
    @Override
    @Transactional
    public boolean save(PersonalDocumentTemplate et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(PersonalDocumentTemplate et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<PersonalDocumentTemplate> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<PersonalDocumentTemplate> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean create(PersonalDocumentTemplate et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getPersonaldocumenttemplateid()),et);
        return true;
    }

    @Override
    public void createBatch(List<PersonalDocumentTemplate> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    public PersonalDocumentTemplate getDraft(PersonalDocumentTemplate et) {
        return et;
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<PersonalDocumentTemplate> searchDefault(PersonalDocumentTemplateSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<PersonalDocumentTemplate> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<PersonalDocumentTemplate>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }





    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<PersonalDocumentTemplate> getPersonaldocumenttemplateByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<PersonalDocumentTemplate> getPersonaldocumenttemplateByEntities(List<PersonalDocumentTemplate> entities) {
        List ids =new ArrayList();
        for(PersonalDocumentTemplate entity : entities){
            Serializable id=entity.getPersonaldocumenttemplateid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



