package cn.ibizlab.businesscentral.core.base.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.base.domain.SlaKpiInstance;
import cn.ibizlab.businesscentral.core.base.filter.SlaKpiInstanceSearchContext;
import cn.ibizlab.businesscentral.core.base.service.ISlaKpiInstanceService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.base.mapper.SlaKpiInstanceMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[SLA KPI 实例] 服务对象接口实现
 */
@Slf4j
@Service("SlaKpiInstanceServiceImpl")
public class SlaKpiInstanceServiceImpl extends ServiceImpl<SlaKpiInstanceMapper, SlaKpiInstance> implements ISlaKpiInstanceService {

    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.service.service.IIncidentService incidentService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.IAccountService accountService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.IBusinessUnitService businessunitService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.ITransactionCurrencyService transactioncurrencyService;

    private int batchSize = 500;

    @Override
    public SlaKpiInstance getDraft(SlaKpiInstance et) {
        return et;
    }

    @Override
    @Transactional
    public boolean create(SlaKpiInstance et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getSlakpiinstanceid()),et);
        return true;
    }

    @Override
    public void createBatch(List<SlaKpiInstance> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    public boolean checkKey(SlaKpiInstance et) {
        return (!ObjectUtils.isEmpty(et.getSlakpiinstanceid()))&&(!Objects.isNull(this.getById(et.getSlakpiinstanceid())));
    }
    @Override
    @Transactional
    public SlaKpiInstance get(String key) {
        SlaKpiInstance et = getById(key);
        if(et==null){
            et=new SlaKpiInstance();
            et.setSlakpiinstanceid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean update(SlaKpiInstance et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("slakpiinstanceid",et.getSlakpiinstanceid())))
            return false;
        CachedBeanCopier.copy(get(et.getSlakpiinstanceid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<SlaKpiInstance> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean save(SlaKpiInstance et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(SlaKpiInstance et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<SlaKpiInstance> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<SlaKpiInstance> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }


	@Override
    public List<SlaKpiInstance> selectByRegarding(String accountid) {
        return baseMapper.selectByRegarding(accountid);
    }

    @Override
    public void removeByRegarding(String accountid) {
        this.remove(new QueryWrapper<SlaKpiInstance>().eq("regarding",accountid));
    }

	@Override
    public List<SlaKpiInstance> selectByOwningbusinessunit(String businessunitid) {
        return baseMapper.selectByOwningbusinessunit(businessunitid);
    }

    @Override
    public void removeByOwningbusinessunit(String businessunitid) {
        this.remove(new QueryWrapper<SlaKpiInstance>().eq("owningbusinessunit",businessunitid));
    }

	@Override
    public List<SlaKpiInstance> selectByTransactioncurrencyid(String transactioncurrencyid) {
        return baseMapper.selectByTransactioncurrencyid(transactioncurrencyid);
    }

    @Override
    public void removeByTransactioncurrencyid(String transactioncurrencyid) {
        this.remove(new QueryWrapper<SlaKpiInstance>().eq("transactioncurrencyid",transactioncurrencyid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<SlaKpiInstance> searchDefault(SlaKpiInstanceSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<SlaKpiInstance> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<SlaKpiInstance>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }





    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<SlaKpiInstance> getSlakpiinstanceByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<SlaKpiInstance> getSlakpiinstanceByEntities(List<SlaKpiInstance> entities) {
        List ids =new ArrayList();
        for(SlaKpiInstance entity : entities){
            Serializable id=entity.getSlakpiinstanceid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



