package cn.ibizlab.businesscentral.core.base.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.base.domain.Subject;
import cn.ibizlab.businesscentral.core.base.filter.SubjectSearchContext;
import cn.ibizlab.businesscentral.core.base.service.ISubjectService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.base.mapper.SubjectMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[主题] 服务对象接口实现
 */
@Slf4j
@Service("SubjectServiceImpl")
public class SubjectServiceImpl extends ServiceImpl<SubjectMapper, Subject> implements ISubjectService {

    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.service.service.IIncidentService incidentService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.IKnowledgeArticleService knowledgearticleService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.product.service.IProductService productService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.sales.service.ISalesLiteratureService salesliteratureService;

    private cn.ibizlab.businesscentral.core.base.service.ISubjectService subjectService = this;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean save(Subject et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(Subject et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<Subject> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<Subject> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean create(Subject et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getSubjectid()),et);
        return true;
    }

    @Override
    public void createBatch(List<Subject> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    public boolean checkKey(Subject et) {
        return (!ObjectUtils.isEmpty(et.getSubjectid()))&&(!Objects.isNull(this.getById(et.getSubjectid())));
    }
    @Override
    @Transactional
    public boolean update(Subject et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("subjectid",et.getSubjectid())))
            return false;
        CachedBeanCopier.copy(get(et.getSubjectid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<Subject> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    public Subject getDraft(Subject et) {
        return et;
    }

    @Override
    @Transactional
    public Subject get(String key) {
        Subject et = getById(key);
        if(et==null){
            et=new Subject();
            et.setSubjectid(key);
        }
        else{
        }
        return et;
    }


	@Override
    public List<Subject> selectByParentsubject(String subjectid) {
        return baseMapper.selectByParentsubject(subjectid);
    }

    @Override
    public void removeByParentsubject(String subjectid) {
        this.remove(new QueryWrapper<Subject>().eq("parentsubject",subjectid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<Subject> searchDefault(SubjectSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<Subject> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<Subject>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }





    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<Subject> getSubjectByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<Subject> getSubjectByEntities(List<Subject> entities) {
        List ids =new ArrayList();
        for(Subject entity : entities){
            Serializable id=entity.getSubjectid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



