package cn.ibizlab.businesscentral.core.finance.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.finance.domain.FixedMonthlyFiscalCalendar;
import cn.ibizlab.businesscentral.core.finance.filter.FixedMonthlyFiscalCalendarSearchContext;
import cn.ibizlab.businesscentral.core.finance.service.IFixedMonthlyFiscalCalendarService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.finance.mapper.FixedMonthlyFiscalCalendarMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[固定月度会计日历] 服务对象接口实现
 */
@Slf4j
@Service("FixedMonthlyFiscalCalendarServiceImpl")
public class FixedMonthlyFiscalCalendarServiceImpl extends ServiceImpl<FixedMonthlyFiscalCalendarMapper, FixedMonthlyFiscalCalendar> implements IFixedMonthlyFiscalCalendarService {

    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.ITransactionCurrencyService transactioncurrencyService;

    private int batchSize = 500;

    @Override
    public boolean checkKey(FixedMonthlyFiscalCalendar et) {
        return (!ObjectUtils.isEmpty(et.getUserfiscalcalendarid()))&&(!Objects.isNull(this.getById(et.getUserfiscalcalendarid())));
    }
    @Override
    @Transactional
    public boolean create(FixedMonthlyFiscalCalendar et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getUserfiscalcalendarid()),et);
        return true;
    }

    @Override
    public void createBatch(List<FixedMonthlyFiscalCalendar> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    public FixedMonthlyFiscalCalendar getDraft(FixedMonthlyFiscalCalendar et) {
        return et;
    }

    @Override
    @Transactional
    public boolean update(FixedMonthlyFiscalCalendar et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("userfiscalcalendarid",et.getUserfiscalcalendarid())))
            return false;
        CachedBeanCopier.copy(get(et.getUserfiscalcalendarid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<FixedMonthlyFiscalCalendar> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public FixedMonthlyFiscalCalendar get(String key) {
        FixedMonthlyFiscalCalendar et = getById(key);
        if(et==null){
            et=new FixedMonthlyFiscalCalendar();
            et.setUserfiscalcalendarid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean save(FixedMonthlyFiscalCalendar et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(FixedMonthlyFiscalCalendar et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<FixedMonthlyFiscalCalendar> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<FixedMonthlyFiscalCalendar> list) {
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<FixedMonthlyFiscalCalendar> selectByTransactioncurrencyid(String transactioncurrencyid) {
        return baseMapper.selectByTransactioncurrencyid(transactioncurrencyid);
    }

    @Override
    public void removeByTransactioncurrencyid(String transactioncurrencyid) {
        this.remove(new QueryWrapper<FixedMonthlyFiscalCalendar>().eq("transactioncurrencyid",transactioncurrencyid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<FixedMonthlyFiscalCalendar> searchDefault(FixedMonthlyFiscalCalendarSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<FixedMonthlyFiscalCalendar> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<FixedMonthlyFiscalCalendar>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }





    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<FixedMonthlyFiscalCalendar> getFixedmonthlyfiscalcalendarByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<FixedMonthlyFiscalCalendar> getFixedmonthlyfiscalcalendarByEntities(List<FixedMonthlyFiscalCalendar> entities) {
        List ids =new ArrayList();
        for(FixedMonthlyFiscalCalendar entity : entities){
            Serializable id=entity.getUserfiscalcalendarid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



