package cn.ibizlab.businesscentral.core.finance.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.finance.domain.InvoiceDetail;
import cn.ibizlab.businesscentral.core.finance.filter.InvoiceDetailSearchContext;
import cn.ibizlab.businesscentral.core.finance.service.IInvoiceDetailService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.finance.mapper.InvoiceDetailMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[发票产品] 服务对象接口实现
 */
@Slf4j
@Service("InvoiceDetailServiceImpl")
public class InvoiceDetailServiceImpl extends ServiceImpl<InvoiceDetailMapper, InvoiceDetail> implements IInvoiceDetailService {


    private cn.ibizlab.businesscentral.core.finance.service.IInvoiceDetailService invoicedetailService = this;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.finance.service.IInvoiceService invoiceService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.product.service.IProductService productService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.sales.service.ISalesOrderDetailService salesorderdetailService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.ITransactionCurrencyService transactioncurrencyService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.IUomService uomService;

    private int batchSize = 500;

    @Override
    public boolean checkKey(InvoiceDetail et) {
        return (!ObjectUtils.isEmpty(et.getInvoicedetailid()))&&(!Objects.isNull(this.getById(et.getInvoicedetailid())));
    }
    @Override
    @Transactional
    public InvoiceDetail get(String key) {
        InvoiceDetail et = getById(key);
        if(et==null){
            et=new InvoiceDetail();
            et.setInvoicedetailid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean update(InvoiceDetail et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("invoicedetailid",et.getInvoicedetailid())))
            return false;
        CachedBeanCopier.copy(get(et.getInvoicedetailid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<InvoiceDetail> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean create(InvoiceDetail et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getInvoicedetailid()),et);
        return true;
    }

    @Override
    public void createBatch(List<InvoiceDetail> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    public InvoiceDetail getDraft(InvoiceDetail et) {
        return et;
    }

    @Override
    @Transactional
    public boolean save(InvoiceDetail et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(InvoiceDetail et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<InvoiceDetail> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<InvoiceDetail> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }


	@Override
    public List<InvoiceDetail> selectByParentbundleidref(String invoicedetailid) {
        return baseMapper.selectByParentbundleidref(invoicedetailid);
    }

    @Override
    public void removeByParentbundleidref(String invoicedetailid) {
        this.remove(new QueryWrapper<InvoiceDetail>().eq("parentbundleidref",invoicedetailid));
    }

	@Override
    public List<InvoiceDetail> selectByInvoiceid(String invoiceid) {
        return baseMapper.selectByInvoiceid(invoiceid);
    }

    @Override
    public void removeByInvoiceid(String invoiceid) {
        this.remove(new QueryWrapper<InvoiceDetail>().eq("invoiceid",invoiceid));
    }

	@Override
    public List<InvoiceDetail> selectByProductid(String productid) {
        return baseMapper.selectByProductid(productid);
    }

    @Override
    public void removeByProductid(String productid) {
        this.remove(new QueryWrapper<InvoiceDetail>().eq("productid",productid));
    }

	@Override
    public List<InvoiceDetail> selectBySalesorderdetailid(String salesorderdetailid) {
        return baseMapper.selectBySalesorderdetailid(salesorderdetailid);
    }

    @Override
    public void removeBySalesorderdetailid(String salesorderdetailid) {
        this.remove(new QueryWrapper<InvoiceDetail>().eq("salesorderdetailid",salesorderdetailid));
    }

	@Override
    public List<InvoiceDetail> selectByTransactioncurrencyid(String transactioncurrencyid) {
        return baseMapper.selectByTransactioncurrencyid(transactioncurrencyid);
    }

    @Override
    public void removeByTransactioncurrencyid(String transactioncurrencyid) {
        this.remove(new QueryWrapper<InvoiceDetail>().eq("transactioncurrencyid",transactioncurrencyid));
    }

	@Override
    public List<InvoiceDetail> selectByUomid(String uomid) {
        return baseMapper.selectByUomid(uomid);
    }

    @Override
    public void removeByUomid(String uomid) {
        this.remove(new QueryWrapper<InvoiceDetail>().eq("uomid",uomid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<InvoiceDetail> searchDefault(InvoiceDetailSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<InvoiceDetail> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<InvoiceDetail>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }





    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<InvoiceDetail> getInvoicedetailByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<InvoiceDetail> getInvoicedetailByEntities(List<InvoiceDetail> entities) {
        List ids =new ArrayList();
        for(InvoiceDetail entity : entities){
            Serializable id=entity.getInvoicedetailid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



