package cn.ibizlab.businesscentral.core.runtime.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.runtime.domain.BulkDeleteOperation;
import cn.ibizlab.businesscentral.core.runtime.filter.BulkDeleteOperationSearchContext;
import cn.ibizlab.businesscentral.core.runtime.service.IBulkDeleteOperationService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.runtime.mapper.BulkDeleteOperationMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[批量删除操作] 服务对象接口实现
 */
@Slf4j
@Service("BulkDeleteOperationServiceImpl")
public class BulkDeleteOperationServiceImpl extends ServiceImpl<BulkDeleteOperationMapper, BulkDeleteOperation> implements IBulkDeleteOperationService {


    private int batchSize = 500;

    @Override
    public boolean checkKey(BulkDeleteOperation et) {
        return (!ObjectUtils.isEmpty(et.getBulkdeleteoperationid()))&&(!Objects.isNull(this.getById(et.getBulkdeleteoperationid())));
    }
    @Override
    @Transactional
    public boolean update(BulkDeleteOperation et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("bulkdeleteoperationid",et.getBulkdeleteoperationid())))
            return false;
        CachedBeanCopier.copy(get(et.getBulkdeleteoperationid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<BulkDeleteOperation> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean save(BulkDeleteOperation et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(BulkDeleteOperation et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<BulkDeleteOperation> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<BulkDeleteOperation> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    public BulkDeleteOperation getDraft(BulkDeleteOperation et) {
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public BulkDeleteOperation get(String key) {
        BulkDeleteOperation et = getById(key);
        if(et==null){
            et=new BulkDeleteOperation();
            et.setBulkdeleteoperationid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean create(BulkDeleteOperation et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getBulkdeleteoperationid()),et);
        return true;
    }

    @Override
    public void createBatch(List<BulkDeleteOperation> list) {
        this.saveBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<BulkDeleteOperation> searchDefault(BulkDeleteOperationSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<BulkDeleteOperation> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<BulkDeleteOperation>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }





    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<BulkDeleteOperation> getBulkdeleteoperationByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<BulkDeleteOperation> getBulkdeleteoperationByEntities(List<BulkDeleteOperation> entities) {
        List ids =new ArrayList();
        for(BulkDeleteOperation entity : entities){
            Serializable id=entity.getBulkdeleteoperationid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



