package cn.ibizlab.businesscentral.core.sales.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.sales.domain.DiscountType;
import cn.ibizlab.businesscentral.core.sales.filter.DiscountTypeSearchContext;
import cn.ibizlab.businesscentral.core.sales.service.IDiscountTypeService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.sales.mapper.DiscountTypeMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[折扣表] 服务对象接口实现
 */
@Slf4j
@Service("DiscountTypeServiceImpl")
public class DiscountTypeServiceImpl extends ServiceImpl<DiscountTypeMapper, DiscountType> implements IDiscountTypeService {

    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.sales.service.IDiscountService discountService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.product.service.IProductPricePevelService productpricepevelService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.ITransactionCurrencyService transactioncurrencyService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean create(DiscountType et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getDiscounttypeid()),et);
        return true;
    }

    @Override
    public void createBatch(List<DiscountType> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    public boolean checkKey(DiscountType et) {
        return (!ObjectUtils.isEmpty(et.getDiscounttypeid()))&&(!Objects.isNull(this.getById(et.getDiscounttypeid())));
    }
    @Override
    public DiscountType getDraft(DiscountType et) {
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean update(DiscountType et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("discounttypeid",et.getDiscounttypeid())))
            return false;
        CachedBeanCopier.copy(get(et.getDiscounttypeid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<DiscountType> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean save(DiscountType et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(DiscountType et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<DiscountType> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<DiscountType> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public DiscountType get(String key) {
        DiscountType et = getById(key);
        if(et==null){
            et=new DiscountType();
            et.setDiscounttypeid(key);
        }
        else{
        }
        return et;
    }


	@Override
    public List<DiscountType> selectByTransactioncurrencyid(String transactioncurrencyid) {
        return baseMapper.selectByTransactioncurrencyid(transactioncurrencyid);
    }

    @Override
    public void removeByTransactioncurrencyid(String transactioncurrencyid) {
        this.remove(new QueryWrapper<DiscountType>().eq("transactioncurrencyid",transactioncurrencyid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<DiscountType> searchDefault(DiscountTypeSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<DiscountType> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<DiscountType>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }





    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<DiscountType> getDiscounttypeByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<DiscountType> getDiscounttypeByEntities(List<DiscountType> entities) {
        List ids =new ArrayList();
        for(DiscountType entity : entities){
            Serializable id=entity.getDiscounttypeid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



