package cn.ibizlab.businesscentral.core.sales.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.sales.domain.SalesLiteratureItem;
import cn.ibizlab.businesscentral.core.sales.filter.SalesLiteratureItemSearchContext;
import cn.ibizlab.businesscentral.core.sales.service.ISalesLiteratureItemService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.sales.mapper.SalesLiteratureItemMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[销售附件] 服务对象接口实现
 */
@Slf4j
@Service("SalesLiteratureItemServiceImpl")
public class SalesLiteratureItemServiceImpl extends ServiceImpl<SalesLiteratureItemMapper, SalesLiteratureItem> implements ISalesLiteratureItemService {

    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.sales.service.ISalesLiteratureService salesliteratureService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean save(SalesLiteratureItem et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(SalesLiteratureItem et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<SalesLiteratureItem> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<SalesLiteratureItem> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public SalesLiteratureItem get(String key) {
        SalesLiteratureItem et = getById(key);
        if(et==null){
            et=new SalesLiteratureItem();
            et.setSalesliteratureitemid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public SalesLiteratureItem getDraft(SalesLiteratureItem et) {
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean update(SalesLiteratureItem et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("salesliteratureitemid",et.getSalesliteratureitemid())))
            return false;
        CachedBeanCopier.copy(get(et.getSalesliteratureitemid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<SalesLiteratureItem> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    public boolean checkKey(SalesLiteratureItem et) {
        return (!ObjectUtils.isEmpty(et.getSalesliteratureitemid()))&&(!Objects.isNull(this.getById(et.getSalesliteratureitemid())));
    }
    @Override
    @Transactional
    public boolean create(SalesLiteratureItem et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getSalesliteratureitemid()),et);
        return true;
    }

    @Override
    public void createBatch(List<SalesLiteratureItem> list) {
        this.saveBatch(list,batchSize);
    }


	@Override
    public List<SalesLiteratureItem> selectBySalesliteratureid(String salesliteratureid) {
        return baseMapper.selectBySalesliteratureid(salesliteratureid);
    }

    @Override
    public void removeBySalesliteratureid(String salesliteratureid) {
        this.remove(new QueryWrapper<SalesLiteratureItem>().eq("salesliteratureid",salesliteratureid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<SalesLiteratureItem> searchDefault(SalesLiteratureItemSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<SalesLiteratureItem> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<SalesLiteratureItem>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }





    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<SalesLiteratureItem> getSalesliteratureitemByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<SalesLiteratureItem> getSalesliteratureitemByEntities(List<SalesLiteratureItem> entities) {
        List ids =new ArrayList();
        for(SalesLiteratureItem entity : entities){
            Serializable id=entity.getSalesliteratureitemid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



