package cn.ibizlab.businesscentral.core.scheduling.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.scheduling.domain.BookableResourceBookingHeader;
import cn.ibizlab.businesscentral.core.scheduling.filter.BookableResourceBookingHeaderSearchContext;
import cn.ibizlab.businesscentral.core.scheduling.service.IBookableResourceBookingHeaderService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.scheduling.mapper.BookableResourceBookingHeaderMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[可预订资源的预订标题] 服务对象接口实现
 */
@Slf4j
@Service("BookableResourceBookingHeaderServiceImpl")
public class BookableResourceBookingHeaderServiceImpl extends ServiceImpl<BookableResourceBookingHeaderMapper, BookableResourceBookingHeader> implements IBookableResourceBookingHeaderService {

    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.scheduling.service.IBookableResourceBookingService bookableresourcebookingService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.ITransactionCurrencyService transactioncurrencyService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean save(BookableResourceBookingHeader et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(BookableResourceBookingHeader et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<BookableResourceBookingHeader> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<BookableResourceBookingHeader> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(BookableResourceBookingHeader et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("bookableresbookingheaderid",et.getBookableresourcebookingheaderid())))
            return false;
        CachedBeanCopier.copy(get(et.getBookableresourcebookingheaderid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<BookableResourceBookingHeader> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public BookableResourceBookingHeader get(String key) {
        BookableResourceBookingHeader et = getById(key);
        if(et==null){
            et=new BookableResourceBookingHeader();
            et.setBookableresourcebookingheaderid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean create(BookableResourceBookingHeader et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getBookableresourcebookingheaderid()),et);
        return true;
    }

    @Override
    public void createBatch(List<BookableResourceBookingHeader> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    public BookableResourceBookingHeader getDraft(BookableResourceBookingHeader et) {
        return et;
    }

    @Override
    public boolean checkKey(BookableResourceBookingHeader et) {
        return (!ObjectUtils.isEmpty(et.getBookableresourcebookingheaderid()))&&(!Objects.isNull(this.getById(et.getBookableresourcebookingheaderid())));
    }

	@Override
    public List<BookableResourceBookingHeader> selectByTransactioncurrencyid(String transactioncurrencyid) {
        return baseMapper.selectByTransactioncurrencyid(transactioncurrencyid);
    }

    @Override
    public void removeByTransactioncurrencyid(String transactioncurrencyid) {
        this.remove(new QueryWrapper<BookableResourceBookingHeader>().eq("transactioncurrencyid",transactioncurrencyid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<BookableResourceBookingHeader> searchDefault(BookableResourceBookingHeaderSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<BookableResourceBookingHeader> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<BookableResourceBookingHeader>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }





    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<BookableResourceBookingHeader> getBookableresourcebookingheaderByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<BookableResourceBookingHeader> getBookableresourcebookingheaderByEntities(List<BookableResourceBookingHeader> entities) {
        List ids =new ArrayList();
        for(BookableResourceBookingHeader entity : entities){
            Serializable id=entity.getBookableresourcebookingheaderid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



