package cn.ibizlab.businesscentral.core.service.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.service.domain.ContractTemplate;
import cn.ibizlab.businesscentral.core.service.filter.ContractTemplateSearchContext;
import cn.ibizlab.businesscentral.core.service.service.IContractTemplateService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.service.mapper.ContractTemplateMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[合同模板] 服务对象接口实现
 */
@Slf4j
@Service("ContractTemplateServiceImpl")
public class ContractTemplateServiceImpl extends ServiceImpl<ContractTemplateMapper, ContractTemplate> implements IContractTemplateService {

    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.service.service.IContractService contractService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean create(ContractTemplate et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getContracttemplateid()),et);
        return true;
    }

    @Override
    public void createBatch(List<ContractTemplate> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean save(ContractTemplate et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(ContractTemplate et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<ContractTemplate> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<ContractTemplate> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    public boolean checkKey(ContractTemplate et) {
        return (!ObjectUtils.isEmpty(et.getContracttemplateid()))&&(!Objects.isNull(this.getById(et.getContracttemplateid())));
    }
    @Override
    @Transactional
    public ContractTemplate get(String key) {
        ContractTemplate et = getById(key);
        if(et==null){
            et=new ContractTemplate();
            et.setContracttemplateid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean update(ContractTemplate et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("contracttemplateid",et.getContracttemplateid())))
            return false;
        CachedBeanCopier.copy(get(et.getContracttemplateid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<ContractTemplate> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    public ContractTemplate getDraft(ContractTemplate et) {
        return et;
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<ContractTemplate> searchDefault(ContractTemplateSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<ContractTemplate> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<ContractTemplate>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }





    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<ContractTemplate> getContracttemplateByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<ContractTemplate> getContracttemplateByEntities(List<ContractTemplate> entities) {
        List ids =new ArrayList();
        for(ContractTemplate entity : entities){
            Serializable id=entity.getContracttemplateid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



