package cn.ibizlab.businesscentral.core.service.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.service.domain.EntitlementChannel;
import cn.ibizlab.businesscentral.core.service.filter.EntitlementChannelSearchContext;
import cn.ibizlab.businesscentral.core.service.service.IEntitlementChannelService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.service.mapper.EntitlementChannelMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[权利渠道] 服务对象接口实现
 */
@Slf4j
@Service("EntitlementChannelServiceImpl")
public class EntitlementChannelServiceImpl extends ServiceImpl<EntitlementChannelMapper, EntitlementChannel> implements IEntitlementChannelService {

    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.service.service.IEntitlementService entitlementService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.ITransactionCurrencyService transactioncurrencyService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean update(EntitlementChannel et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("entitlementchannelid",et.getEntitlementchannelid())))
            return false;
        CachedBeanCopier.copy(get(et.getEntitlementchannelid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EntitlementChannel> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public EntitlementChannel get(String key) {
        EntitlementChannel et = getById(key);
        if(et==null){
            et=new EntitlementChannel();
            et.setEntitlementchannelid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean save(EntitlementChannel et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EntitlementChannel et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EntitlementChannel> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EntitlementChannel> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    public EntitlementChannel getDraft(EntitlementChannel et) {
        return et;
    }

    @Override
    public boolean checkKey(EntitlementChannel et) {
        return (!ObjectUtils.isEmpty(et.getEntitlementchannelid()))&&(!Objects.isNull(this.getById(et.getEntitlementchannelid())));
    }
    @Override
    @Transactional
    public boolean create(EntitlementChannel et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEntitlementchannelid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EntitlementChannel> list) {
        this.saveBatch(list,batchSize);
    }


	@Override
    public List<EntitlementChannel> selectByEntitlementid(String entitlementid) {
        return baseMapper.selectByEntitlementid(entitlementid);
    }

    @Override
    public void removeByEntitlementid(String entitlementid) {
        this.remove(new QueryWrapper<EntitlementChannel>().eq("entitlementid",entitlementid));
    }

	@Override
    public List<EntitlementChannel> selectByTransactioncurrencyid(String transactioncurrencyid) {
        return baseMapper.selectByTransactioncurrencyid(transactioncurrencyid);
    }

    @Override
    public void removeByTransactioncurrencyid(String transactioncurrencyid) {
        this.remove(new QueryWrapper<EntitlementChannel>().eq("transactioncurrencyid",transactioncurrencyid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EntitlementChannel> searchDefault(EntitlementChannelSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EntitlementChannel> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EntitlementChannel>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }





    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EntitlementChannel> getEntitlementchannelByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EntitlementChannel> getEntitlementchannelByEntities(List<EntitlementChannel> entities) {
        List ids =new ArrayList();
        for(EntitlementChannel entity : entities){
            Serializable id=entity.getEntitlementchannelid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



