package cn.ibizlab.businesscentral.core.service.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.service.domain.Incident;
import cn.ibizlab.businesscentral.core.service.filter.IncidentSearchContext;
import cn.ibizlab.businesscentral.core.service.service.IIncidentService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.service.mapper.IncidentMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[案例] 服务对象接口实现
 */
@Slf4j
@Service("IncidentServiceImpl")
public class IncidentServiceImpl extends ServiceImpl<IncidentMapper, Incident> implements IIncidentService {

    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.service.service.IIncidentreSolutionService incidentresolutionService;

    private cn.ibizlab.businesscentral.core.service.service.IIncidentService incidentService = this;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.service.service.IKnowledgeArticleIncidentService knowledgearticleincidentService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.sales.service.ILeadService leadService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.IContactService contactService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.service.service.IContractDetailService contractdetailService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.service.service.IContractService contractService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.service.service.IEntitlementService entitlementService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.product.service.IProductService productService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.ISlaKpiInstanceService slakpiinstanceService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.ISlaService slaService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.ISubjectService subjectService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.ITransactionCurrencyService transactioncurrencyService;

    private int batchSize = 500;

    @Override
    public Incident getDraft(Incident et) {
        return et;
    }

    @Override
    @Transactional
    public boolean update(Incident et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("incidentid",et.getIncidentid())))
            return false;
        CachedBeanCopier.copy(get(et.getIncidentid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<Incident> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean create(Incident et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getIncidentid()),et);
        return true;
    }

    @Override
    public void createBatch(List<Incident> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean save(Incident et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(Incident et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<Incident> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<Incident> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    public boolean checkKey(Incident et) {
        return (!ObjectUtils.isEmpty(et.getIncidentid()))&&(!Objects.isNull(this.getById(et.getIncidentid())));
    }
    @Override
    @Transactional
    public Incident get(String key) {
        Incident et = getById(key);
        if(et==null){
            et=new Incident();
            et.setIncidentid(key);
        }
        else{
        }
        return et;
    }


	@Override
    public List<Incident> selectByPrimarycontactid(String contactid) {
        return baseMapper.selectByPrimarycontactid(contactid);
    }

    @Override
    public void removeByPrimarycontactid(String contactid) {
        this.remove(new QueryWrapper<Incident>().eq("primarycontactid",contactid));
    }

	@Override
    public List<Incident> selectByResponsiblecontactid(String contactid) {
        return baseMapper.selectByResponsiblecontactid(contactid);
    }

    @Override
    public void removeByResponsiblecontactid(String contactid) {
        this.remove(new QueryWrapper<Incident>().eq("responsiblecontactid",contactid));
    }

	@Override
    public List<Incident> selectByContractdetailid(String contractdetailid) {
        return baseMapper.selectByContractdetailid(contractdetailid);
    }

    @Override
    public void removeByContractdetailid(String contractdetailid) {
        this.remove(new QueryWrapper<Incident>().eq("contractdetailid",contractdetailid));
    }

	@Override
    public List<Incident> selectByContractid(String contractid) {
        return baseMapper.selectByContractid(contractid);
    }

    @Override
    public void removeByContractid(String contractid) {
        this.remove(new QueryWrapper<Incident>().eq("contractid",contractid));
    }

	@Override
    public List<Incident> selectByEntitlementid(String entitlementid) {
        return baseMapper.selectByEntitlementid(entitlementid);
    }

    @Override
    public void removeByEntitlementid(String entitlementid) {
        this.remove(new QueryWrapper<Incident>().eq("entitlementid",entitlementid));
    }

	@Override
    public List<Incident> selectByExistingcase(String incidentid) {
        return baseMapper.selectByExistingcase(incidentid);
    }

    @Override
    public void removeByExistingcase(String incidentid) {
        this.remove(new QueryWrapper<Incident>().eq("existingcase",incidentid));
    }

	@Override
    public List<Incident> selectByMasterid(String incidentid) {
        return baseMapper.selectByMasterid(incidentid);
    }

    @Override
    public void removeByMasterid(String incidentid) {
        this.remove(new QueryWrapper<Incident>().eq("masterid",incidentid));
    }

	@Override
    public List<Incident> selectByParentcaseid(String incidentid) {
        return baseMapper.selectByParentcaseid(incidentid);
    }

    @Override
    public void removeByParentcaseid(String incidentid) {
        this.remove(new QueryWrapper<Incident>().eq("parentcaseid",incidentid));
    }

	@Override
    public List<Incident> selectByProductid(String productid) {
        return baseMapper.selectByProductid(productid);
    }

    @Override
    public void removeByProductid(String productid) {
        this.remove(new QueryWrapper<Incident>().eq("productid",productid));
    }

	@Override
    public List<Incident> selectByFirstresponsebykpiid(String slakpiinstanceid) {
        return baseMapper.selectByFirstresponsebykpiid(slakpiinstanceid);
    }

    @Override
    public void removeByFirstresponsebykpiid(String slakpiinstanceid) {
        this.remove(new QueryWrapper<Incident>().eq("firstresponsebykpiid",slakpiinstanceid));
    }

	@Override
    public List<Incident> selectByResolvebykpiid(String slakpiinstanceid) {
        return baseMapper.selectByResolvebykpiid(slakpiinstanceid);
    }

    @Override
    public void removeByResolvebykpiid(String slakpiinstanceid) {
        this.remove(new QueryWrapper<Incident>().eq("resolvebykpiid",slakpiinstanceid));
    }

	@Override
    public List<Incident> selectBySlaid(String slaid) {
        return baseMapper.selectBySlaid(slaid);
    }

    @Override
    public void removeBySlaid(String slaid) {
        this.remove(new QueryWrapper<Incident>().eq("slaid",slaid));
    }

	@Override
    public List<Incident> selectBySubjectid(String subjectid) {
        return baseMapper.selectBySubjectid(subjectid);
    }

    @Override
    public void removeBySubjectid(String subjectid) {
        this.remove(new QueryWrapper<Incident>().eq("subjectid",subjectid));
    }

	@Override
    public List<Incident> selectByTransactioncurrencyid(String transactioncurrencyid) {
        return baseMapper.selectByTransactioncurrencyid(transactioncurrencyid);
    }

    @Override
    public void removeByTransactioncurrencyid(String transactioncurrencyid) {
        this.remove(new QueryWrapper<Incident>().eq("transactioncurrencyid",transactioncurrencyid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<Incident> searchDefault(IncidentSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<Incident> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<Incident>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }





    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<Incident> getIncidentByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<Incident> getIncidentByEntities(List<Incident> entities) {
        List ids =new ArrayList();
        for(Incident entity : entities){
            Serializable id=entity.getIncidentid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



